#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <m17n-gui.h>

void
print_font (MFont *font)
{
  MSymbol family, language, script, otf;

  family = mfont_get_prop (font, Mfamily);
  language = mfont_get_prop (font, Mlanguage);
  script = mfont_get_prop (font, Mscript);
  otf = mfont_get_prop (font, Motf);

  if (family)
    printf ("%s", msymbol_name (family));
  if (script)
    printf (":script=%s", msymbol_name (script));
  if (language)
    printf (":lang=%s", msymbol_name (language));
  if (otf)
    printf (":otf=%s", msymbol_name (otf));
}

int
main (int argc, char **argv)
{
  MFrame *frame;
  MFontset *fontset;
  MPlist *plist, *pl, *p, *p0;

  if (argc < 2
      || strcmp ("-h", argv[1]) == 0 || strcmp ("--help", argv[1]) == 0)
    {
      printf ("Usage: check-fs FONTSET-NAME\n");
      exit (1);
    }

  M17N_INIT ();
  plist = mplist ();
  mplist_add (plist, Mdevice, NULL);
  frame = mframe (plist);
  m17n_object_unref (plist);

  fontset = mfontset (argv[1]);
  for (plist = mfontset_lookup (fontset, Mt, Mnil, Mnil);
       plist &&  mplist_key (plist) != Mnil; plist = mplist_next (plist))
    {
      MSymbol script = mplist_key (plist);

      printf ("\n(%s", msymbol_name (script));
      for (pl = mfontset_lookup (fontset, script, Mt, Mnil);
	   pl && mplist_key (pl) != Mnil; pl = mplist_next (pl))
	{
	  MSymbol language = mplist_key (pl);

	  if (language == Mt)
	    language = Mnil;
	  printf ("\n  (%s", msymbol_name (language));
	  for (p = mfontset_lookup (fontset, script, language, Mnil);
	       p && mplist_key (p) != Mnil; p = mplist_next (p))
	    {
	      MSymbol flt = mplist_key (p);
	      MFont *font = mplist_value (p);

	      printf ("\n    (");
	      print_font (font);
	      if (flt != Mt)
		printf (" (with flt %s)", msymbol_name (flt));
	      for (p0 = mfont_list (frame, font, Mnil, 0);
		   p0 && mplist_key (p0) != Mnil; p0 = mplist_next (p0))
		if (! mplist_find_by_key (mplist_next (p0), mplist_key (p0)))
		  printf ("\n      %s", msymbol_name (mplist_key (p0)));
	      printf (")");
	    }
	  printf (")");
	}
      printf (")");
    }
  printf ("\n");

  m17n_object_unref (frame);
  M17N_FINI ();
  exit (0);
}
